'use strict';

var _ALIGN_TYPE_TO_POSITI;

var _require = require('./imageServiceConstants'),
    alignTypes = _require.alignTypes,
    fittingTypes = _require.fittingTypes;

var ALIGN_TYPE_TO_POSITION = (_ALIGN_TYPE_TO_POSITI = {}, _ALIGN_TYPE_TO_POSITI[alignTypes.CENTER] = '50% 50%', _ALIGN_TYPE_TO_POSITI[alignTypes.TOP_LEFT] = '0% 0%', _ALIGN_TYPE_TO_POSITI[alignTypes.TOP_RIGHT] = '100% 0%', _ALIGN_TYPE_TO_POSITI[alignTypes.TOP] = '50% 0%', _ALIGN_TYPE_TO_POSITI[alignTypes.BOTTOM_LEFT] = '0% 100%', _ALIGN_TYPE_TO_POSITI[alignTypes.BOTTOM_RIGHT] = '100% 100%', _ALIGN_TYPE_TO_POSITI[alignTypes.BOTTOM] = '50% 100%', _ALIGN_TYPE_TO_POSITI[alignTypes.RIGHT] = '100% 50%', _ALIGN_TYPE_TO_POSITI[alignTypes.LEFT] = '0% 50%', _ALIGN_TYPE_TO_POSITI);
var POSITION_TO_ALIGN_TYPE = Object.entries(ALIGN_TYPE_TO_POSITION).reduce(function (acc, _ref) {
  var align = _ref[0],
      position = _ref[1];
  acc[position] = align;
  return acc;
}, {});
var TILE_FITTING_TYPES = [fittingTypes.TILE, fittingTypes.TILE_HORIZONTAL, fittingTypes.TILE_VERTICAL, fittingTypes.LEGACY_BG_FIT_AND_TILE, fittingTypes.LEGACY_BG_FIT_AND_TILE_HORIZONTAL, fittingTypes.LEGACY_BG_FIT_AND_TILE_VERTICAL];
var NON_SCALING_FITTING_TYPES = [fittingTypes.LEGACY_ORIGINAL_SIZE, fittingTypes.ORIGINAL_SIZE, fittingTypes.LEGACY_BG_NORMAL];

function getIsFakeTile(fittingType, src, _ref2) {
  var targetWidth = _ref2.width,
      targetHeight = _ref2.height;
  return fittingType === fittingTypes.TILE && src.width > targetWidth && src.height > targetHeight;
}
/**
 *
 * @param {string}                  fittingType      imageServicesTypes.fittingTypes
 * @param {ImageTransformSource}    src              source image
 * @param {Dimensions}              target           target container dimensions
 * @param {boolean}                 [isSEOBot=false] whether it is a render for a bot request
 * @returns {Dimensions} scaled dimensions
 */


function getScaledDimensions(fittingType, src, _ref3, isSEOBot) {
  var width = _ref3.width,
      height = _ref3.height;

  if (isSEOBot === void 0) {
    isSEOBot = false;
  }

  if (isSEOBot) {
    return {
      width: width,
      height: height
    };
  }

  var isScalable = !NON_SCALING_FITTING_TYPES.includes(fittingType);
  var isFakeTile = getIsFakeTile(fittingType, src, {
    width: width,
    height: height
  });
  var isTile = !isFakeTile && TILE_FITTING_TYPES.includes(fittingType);

  var _width = isTile ? src.width : width;

  var _height = isTile ? src.height : height;

  var scaleFactor = isScalable ? getScaleFactorByWidth(_width) : 1;
  return {
    width: isFakeTile ? 1920 : _width * scaleFactor,
    height: _height * scaleFactor
  };
}
/**
 *
 * @param {string} fittingType  imageServicesTypes.fittingTypes
 * @param {boolean} isFakeTile
 * @returns {string} fittingType
 */


function getConvertedFitting(fittingType, isFakeTile) {
  var isTile = TILE_FITTING_TYPES.includes(fittingType) && !isFakeTile;
  var isFill = fittingType === fittingTypes.SCALE_TO_FILL;
  return isFill || isTile ? fittingTypes.SCALE_TO_FIT : fittingType;
}
/**
 *
 * @param {ImageTransformSource}    src                 source image
 * @param {number || ''}            width
 * @param {number || ''}            height
 * @returns {{number}, {number}}    width, height
 */


function validateTargetDimensions(src, _ref4) {
  var width = _ref4.width,
      height = _ref4.height;

  if (!width || !height) {
    var _width = width || Math.min(980, src.width);

    var heightRatio = _width / src.width;
    return {
      width: _width,
      height: height || src.height * heightRatio
    };
  }

  return {
    width: width,
    height: height
  };
}
/**
 *
 * @param {number} width
 * @return {number}
 */


function getScaleFactorByWidth(width) {
  if (width > 900) {
    return 0.25;
  } else if (width > 500) {
    return 0.3;
  } else if (width > 200) {
    return 0.4;
  }

  return 1;
}
/**
 *
 * @param {number} width
 * @param {string} fittingType imageServicesTypes.fittingTypes
 * @param {boolean} isSEOBot
 * @return {number}
 */


function getBlurValue(width, fittingType, isSEOBot) {
  if (isSEOBot) {
    return 0;
  }

  if (TILE_FITTING_TYPES.includes(fittingType)) {
    return 1;
  }

  if (width > 200) {
    return 2;
  }

  return 3;
}
/**
 *
 * @param {string}               fittingType
 * @param {ImageTransformSource} src                  source image
 * @param {Dimensions}           target               target element
 * @param {string}               [alignment='center']
 * @returns {{img}, {container}}
 */


function getCSSOverrides(fittingType, src, target, alignment) {
  if (alignment === void 0) {
    alignment = 'center';
  }

  var returnValue = {
    img: {},
    container: {}
  };

  if (fittingType === fittingTypes.SCALE_TO_FILL) {
    var alignTypeFromFocalPoint = src.focalPoint && convertFocalPointToAlignType(src.focalPoint);
    var alignType = alignTypeFromFocalPoint || alignment;

    if (src.focalPoint && !alignTypeFromFocalPoint) {
      returnValue.img = {
        objectPosition: convertFillFocalToPosition(src, target, src.focalPoint)
      };
    } else {
      returnValue.img = {
        objectPosition: ALIGN_TYPE_TO_POSITION[alignType]
      };
    }
  } else if ([fittingTypes.LEGACY_ORIGINAL_SIZE, fittingTypes.ORIGINAL_SIZE].includes(fittingType)) {
    returnValue.img = {
      objectFit: 'none',
      top: 'auto',
      left: 'auto',
      right: 'auto',
      bottom: 'auto'
    };
  } else if (TILE_FITTING_TYPES.includes(fittingType)) {
    returnValue.container = {
      backgroundSize: src.width + "px " + src.height + "px"
    };
  }

  return returnValue;
}
/**
 * Try to convert focal point value to 9 grid alignment value
 *
 * @param {Point}    focalPoint
 * @returns {string} align type
 */


function convertFocalPointToAlignType(focalPoint) {
  var position = focalPoint.x + "% " + focalPoint.y + "%";
  return POSITION_TO_ALIGN_TYPE[position] || '';
}
/**
 *
 * @param {Dimensions} src source dimensions
 * @param {Dimensions} target target dimensions
 * @param {Point} focalPoint x/y as 0-100 percentages
 * @returns {string} in 'x% y%' format
 */


function convertFillFocalToPosition(src, target, focalPoint) {
  var sW = src.width,
      sH = src.height;
  var tW = target.width,
      tH = target.height;
  var fpX = focalPoint.x,
      fpY = focalPoint.y;

  if (!tW || !tH) {
    return fpX + "% " + fpY + "%";
  }

  var fillScaleFactor = Math.max(tW / sW, tH / sH);
  var imgScaledW = sW * fillScaleFactor;
  var imgScaledH = sH * fillScaleFactor;
  var x = Math.max(0, Math.min(imgScaledW - tW, imgScaledW * (fpX / 100) - tW / 2));
  var y = Math.max(0, Math.min(imgScaledH - tH, imgScaledH * (fpY / 100) - tH / 2));
  var posX = x && Math.floor(x / (imgScaledW - tW) * 100);
  var posY = y && Math.floor(y / (imgScaledH - tH) * 100);
  return posX + "% " + posY + "%";
}

module.exports.validateTargetDimensions = validateTargetDimensions;
module.exports.getScaledDimensions = getScaledDimensions;
module.exports.getConvertedFitting = getConvertedFitting;
module.exports.getIsFakeTile = getIsFakeTile;
module.exports.getBlurValue = getBlurValue;
module.exports.getCSSOverrides = getCSSOverrides;
/**
 * @typedef Dimensions
 * @property {number} width
 * @property {number} height
 *
 * @typedef Point
 * @property {number} x
 * @property {number} y
 */
//# sourceMappingURL=imagePlaceholderUtils.js.map